import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.abc.user_warnings_repository_abc import UserWarningsRepositoryABC
from bot_data.model.server_config import ServerConfig
from bot_data.model.user import User
from bot_data.model.user_warnings import UserWarnings
from modules.level.service.level_service import LevelService
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class UserWarningsService:
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        db: DatabaseContextABC,
        bot: DiscordBotServiceABC,
        warnings: UserWarningsRepositoryABC,
        users: UserRepositoryABC,
        servers: ServerRepositoryABC,
        levels: LevelRepositoryABC,
        level_service: LevelService,
        message_service: MessageServiceABC,
        t: TranslatePipe,
        permissions: PermissionServiceABC,
    ):
        self._config = config
        self._logger = logger
        self._db = db
        self._bot = bot
        self._warnings = warnings
        self._users = users
        self._servers = servers
        self._levels = levels
        self._level_service = level_service
        self._message_service = message_service
        self._t = t
        self._permissions = permissions

    async def notify_team(self, member: discord.Member, description: str, removed=False):
        try:
            settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{member.guild.id}")
            channel = member.guild.get_channel(settings.team_channel_id)
            if removed:
                translation = self._t.transform("modules.base.warnings.team_removed").format(
                    description, member.mention
                )
            else:
                translation = self._t.transform("modules.base.warnings.team_warned").format(member.mention, description)

            self._bot.loop.create_task(
                self._message_service.send_channel_message(channel, translation, is_persistent=True)
            )
        except Exception as e:
            self._logger.error(__name__, f"Team notification for user warning failed!", e)

    async def notify_user(self, member: discord.Member, message: str):
        try:
            # run as task to keep the interaction alive
            self._bot.loop.create_task(self._message_service.send_dm_message(message, member))
        except Exception as e:
            self._logger.error(__name__, f"User notification for user warning failed!", e)

    async def check_for_warnings(self, member: discord.Member, user: User):
        existing_warnings = self._warnings.get_user_warnings_by_user_id(user.id)

        if existing_warnings.count() == 1:
            await self.notify_user(member, self._t.transform("modules.base.warnings.first"))

        elif existing_warnings.count() == 2:
            server = self._servers.get_server_by_discord_id(member.guild.id)
            user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)
            level = self._level_service.get_level(user)
            levels = self._levels.get_levels_by_server_id(server.id).order_by(lambda l: l.min_xp)

            new_level = levels.where(lambda l: l.min_xp < level.min_xp).last_or_default()
            if new_level is not None:
                user.xp = new_level.min_xp
                self._users.update_user(user)
                self._db.save_changes()
            await self.notify_user(member, self._t.transform("modules.base.warnings.second"))

        elif existing_warnings.count() == 3:
            server = self._servers.get_server_by_discord_id(member.guild.id)
            user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)
            levels = self._levels.get_levels_by_server_id(server.id)

            new_level = levels.where(lambda l: l.min_xp > 0).order_by(lambda l: l.min_xp).last_or_default()
            if new_level is not None:
                user.xp = new_level.min_xp
                self._users.update_user(user)
                self._db.save_changes()
            await self.notify_user(member, self._t.transform("modules.base.warnings.third"))

        elif existing_warnings.count() >= 4:
            user.xp = 0
            self._users.update_user(user)
            self._db.save_changes()
            await self.notify_team(
                member,
                self._t.transform("modules.base.warnings.kick").format(member.mention),
            )
            await member.kick()

    async def _notify_after_add(self, member: discord.Member, warning: UserWarnings):
        server = self._servers.get_server_by_discord_id(member.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)

        await self.notify_user(
            member,
            self._t.transform("modules.base.warnings.warned").format(warning.description),
        )
        await self.notify_team(member, warning.description)
        await self.check_for_warnings(member, user)

    def _add_warnings(self, member: discord.Member, description: str, author_id: int = None):
        server = self._servers.get_server_by_discord_id(member.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)

        author = None
        if author_id is not None:
            author = self._users.get_user_by_discord_id_and_server_id(author_id, server.id)

        warning = UserWarnings(description, user, author)
        self._warnings.add_user_warnings(warning)
        self._db.save_changes()
        return warning

    def add_warnings(self, member: discord.Member, description: str, author_id: int = None):
        warning = self._add_warnings(member, description, author_id)
        self._bot.loop.create_task(self._notify_after_add(member, warning))

    async def add_warnings_async(self, member: discord.Member, description: str, author_id: int = None):
        warning = self._add_warnings(member, description, author_id)
        await self._notify_after_add(member, warning)

    async def _notify_after_remove(self, warning: UserWarnings):
        guild = self._bot.get_guild(warning.user.server.discord_id)
        member = guild.get_member(warning.user.discord_id)
        await self.notify_user(
            member,
            self._t.transform("modules.base.warnings.removed").format(warning.description),
        )
        await self.notify_team(member, warning.description, removed=True)

    def _remove_warnings(self, id: int):
        warning = self._warnings.get_user_warnings_by_id(id)
        self._warnings.delete_user_warnings(warning)
        self._db.save_changes()
        return warning

    def remove_warnings(self, id: int):
        warning = self._remove_warnings(id)
        self._bot.loop.create_task(self._notify_after_remove(warning))

    async def remove_warnings_async(self, id: int):
        warning = self._remove_warnings(id)
        await self._notify_after_remove(warning)
